@@ -30,12 +30,12 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -60,7 +60,9 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                     Always False, kept for consistency, ignored.
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
-                   If slx_lags>0, the specification becomes of the SDEM type.
+                   If slx_lags>0, the specification becomes of the SLX-Error type.
+    slx_vars     : either "all" (default) or list of booleans to select x variables
+                   to be lagged       
     vm           : boolean
                    If True, include variance-covariance matrix in summary
                    results
@@ -80,7 +82,9 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                    Name of regime variable for use in the output
     latex        : boolean
                    Specifies if summary is to be printed in latex format
-
+    hard_bound   : boolean
+                   If true, raises an exception if the estimated spatial
+                   autoregressive parameter is outside the maximum/minimum bounds.
     Attributes
     ----------
     output       : dataframe
@@ -286,6 +290,7 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         regime_err_sep=False,
         regime_lag_sep=False,
         slx_lags=0,
+        slx_vars="all",
         cores=False,
         name_ds=None,
         name_regimes=None,
@@ -293,17 +298,17 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
     ):
 
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
         x_constant, name_x, warn = USER.check_constant(
             x, name_x, just_rem=True)
         set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
 
         if slx_lags >0:
-            lag_x = get_lags(w, x_constant, slx_lags)
-            x_constant = np.hstack((x_constant, lag_x))
-            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
+            x_constant,name_x = USER.flex_wx(w,x=x_constant,name_x=name_x,constant=False,
+                                                slx_lags=slx_lags,slx_vars=slx_vars)
+            set_warn(self,"WX is computed using the complete W, i.e. not trimmed by regimes.")
 
         self.name_x_r = USER.set_name_x(name_x, x_constant)
 
@@ -312,6 +317,7 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_w = USER.set_name_w(name_w, w)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.n = n
         self.y = y
@@ -331,15 +337,15 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                     "All coefficients must vary across regimes if regime_err_sep = True."
                 )
         else:
-            x_constant = sphstack(
+            x_constant1 = sphstack(
                 np.ones((x_constant.shape[0], 1)), x_constant)
-            name_x = USER.set_name_x(name_x, x_constant)
-            self.x, self.name_x, x_rlist = REGI.Regimes_Frame.__init__(
+            name_x = USER.set_name_x(name_x, x_constant, constant=True)
+            self.x, self.name_x, xtype, x_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 x_constant,
                 regimes,
-                constant_regi=None,
-                cols2regi=cols2regi,
+                constant_regi=constant_regi,
+                cols2regi=cols2regi[1:],
                 names=name_x,
                 rlist=True,
             )
@@ -347,7 +353,7 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self.k = ols.x.shape[1]
             moments = _momentsGM_Error(w, ols.u)
             lambda1 = optim_moments(moments)
-            xs = get_spFilter(w, lambda1, x_constant)
+            xs = get_spFilter(w, lambda1, x_constant1)
             ys = get_spFilter(w, lambda1, y)
             xs = REGI.Regimes_Frame.__init__(
                 self, xs, regimes, constant_regi=None, cols2regi=cols2regi
@@ -364,14 +370,14 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             if slx_lags == 0:
                 self.title = "GM SPATIALLY WEIGHTED MODEL - REGIMES"
             else:
-                self.title = "GM SPATIALLY WEIGHTED MODEL + SLX (SDEM) - REGIMES"       
+                self.title = "GM SPATIALLY WEIGHTED MODEL + SLX (SLX-Error) - REGIMES"       
             self.name_x.append("lambda")
             self.kf += 1
             self.chow = REGI.Chow(self)
             self._cache = {}
             self.output = pd.DataFrame(self.name_x,
                                        columns=['var_names'])
-            self.output['var_type'] = ['x']*(len(self.name_x)-1)+['lambda']
+            self.output['var_type'] = xtype + ['lambda']
             self.output['regime'] = x_rlist + ['_Global']
             self.output['equation'] = 0
             output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
@@ -409,7 +415,7 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                         name_x + ["lambda"],
                         self.name_w,
                         self.name_regimes,
-                        slx_lags,
+                        #slx_lags,
                     ),
                 )
             else:
@@ -425,7 +431,7 @@ class GM_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                         name_x + ["lambda"],
                         self.name_w,
                         self.name_regimes,
-                        slx_lags,
+                        #slx_lags,
                     )
                 )
 
@@ -501,19 +507,19 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    yend         : array
+    yend         : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    endogenous variable
-    q            : array
+    q            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    external exogenous variable to use as instruments (note:
                    this should not contain any variables from x)
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -538,7 +544,7 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                     Always False, kept for consistency, ignored.
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
-                   If slx_lags>0, the specification becomes of the SDEM type.
+                   If slx_lags>0, the specification becomes of the SLX-Error type.
     vm           : boolean
                    If True, include variance-covariance matrix in summary
                    results
@@ -562,7 +568,9 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                    Name of regime variable for use in the output
     latex        : boolean
                    Specifies if summary is to be printed in latex format
-
+    hard_bound   : boolean
+                   If true, raises an exception if the estimated spatial
+                   autoregressive parameter is outside the maximum/minimum bounds.
     Attributes
     ----------
     output       : dataframe
@@ -813,8 +821,9 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
     ):
 
         n = USER.check_arrays(y, x, yend, q)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
         x_constant, name_x, warn = USER.check_constant(
             x, name_x, just_rem=True)
         set_warn(self, warn)
@@ -828,6 +837,7 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         self.constant_regi = constant_regi
         self.cols2regi = cols2regi
         self.name_ds = USER.set_name_ds(name_ds)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.name_w = USER.set_name_w(name_w, w)
         self.n = n
@@ -871,22 +881,22 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                     "All coefficients must vary across regimes if regime_err_sep = True."
                 )
         else:
-            x_constant = sphstack(
+            x_constant1 = sphstack(
                 np.ones((x_constant.shape[0], 1)), x_constant)
-            name_x = USER.set_name_x(name_x, x_constant)
-            q, name_q = REGI.Regimes_Frame.__init__(
+            name_x = USER.set_name_x(name_x, x_constant, constant=True)
+            q, name_q, _ = REGI.Regimes_Frame.__init__(
                 self, q, regimes, constant_regi=None, cols2regi="all", names=name_q
             )
-            x, name_x, x_rlist = REGI.Regimes_Frame.__init__(
+            x, name_x, xtype, x_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 x_constant,
                 regimes,
-                constant_regi=None,
-                cols2regi=cols2regi,
+                constant_regi=constant_regi,
+                cols2regi=cols2regi[1:],
                 names=name_x,
                 rlist=True,
             )
-            yend2, name_yend, yend_rlist = REGI.Regimes_Frame.__init__(
+            yend2, name_yend, xtypeyd, yend_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 yend,
                 regimes,
@@ -903,7 +913,7 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self.yend, self.z = tsls.yend, tsls.z
             moments = _momentsGM_Error(w, tsls.u)
             lambda1 = optim_moments(moments)
-            xs = get_spFilter(w, lambda1, x_constant)
+            xs = get_spFilter(w, lambda1, x_constant1)
             xs = REGI.Regimes_Frame.__init__(
                 self, xs, regimes, constant_regi=None, cols2regi=cols2regi
             )[0]
@@ -937,15 +947,14 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self._cache = {}
             self.output = pd.DataFrame(self.name_z,
                                        columns=['var_names'])
-            self.output['var_type'] = [
-                'x'] * len(self.name_x) + ['yend'] * len(self.name_yend) + ['lambda']
+            self.output['var_type'] = xtype + xtypeyd + ['lambda']
             self.output['regime'] = x_rlist + yend_rlist + ['_Global']
             self.output['equation'] = 0
             if summ:
                 if slx_lags == 0:
                     self.title = ("GM SPATIALLY WEIGHTED 2SLS - REGIMES")
                 else:
-                    self.title = ("GM SPATIALLY WEIGHTED 2SLS WITH SLX (SDEM) - REGIMES")
+                    self.title = ("GM SPATIALLY WEIGHTED 2SLS WITH SLX (SLX-Error) - REGIMES")
                 output(reg=self, vm=vm, robust=False,
                        other_end=False, latex=latex)
 
@@ -1106,11 +1115,11 @@ class GM_Endog_Error_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                     regi_ids[r],
                 ] = results[r].e_pred
                 results[r].other_top = _spat_pseudo_r2(results[r])
-                v_type = ['x'] * len(results[r].name_x) + ['yend'] * \
+                v_type = ['o'] + ['x'] * (len(results[r].name_x)-1) + ['yend'] * \
                     (len(results[r].name_yend) - 1) + ['rho', 'lambda']
             else:
                 results[r].other_top = ""
-                v_type = ['x'] * len(results[r].name_x) + ['yend'] * \
+                v_type = ['o'] + ['x'] * (len(results[r].name_x)-1) + ['yend'] * \
                     len(results[r].name_yend) + ['lambda']
             self.output = pd.concat([self.output, pd.DataFrame({'var_names': results[r].name_z,
                                                                 'var_type': v_type,
@@ -1130,12 +1139,12 @@ class GM_Combo_Regimes(GM_Endog_Error_Regimes, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     yend         : array
@@ -1200,7 +1209,9 @@ class GM_Combo_Regimes(GM_Endog_Error_Regimes, REGI.Regimes_Frame):
                    Name of regime variable for use in the output
     latex        : boolean
                    Specifies if summary is to be printed in latex format
-
+    hard_bound   : boolean
+                   If true, raises an exception if the estimated spatial
+                   autoregressive parameter is outside the maximum/minimum bounds.
     Attributes
     ----------
     output       : dataframe
@@ -1491,8 +1502,9 @@ class GM_Combo_Regimes(GM_Endog_Error_Regimes, REGI.Regimes_Frame):
             set_warn(self, "regime_err_sep set to False when regime_lag_sep=False.")                
             regime_err_sep = False
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
         x_constant, name_x, warn = USER.check_constant(
             x, name_x, just_rem=True)
         set_warn(self, warn)
@@ -1500,6 +1512,7 @@ class GM_Combo_Regimes(GM_Endog_Error_Regimes, REGI.Regimes_Frame):
         self.name_y = USER.set_name_y(name_y)
         name_yend = USER.set_name_yend(name_yend, yend)
         name_q = USER.set_name_q(name_q, q)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
 
         if regime_err_sep and any(col != True for col in cols2regi):
             set_warn(self, "All coefficients must vary across regimes if regime_err_sep = True, so setting cols2regi = 'all'.")
@@ -1538,7 +1551,6 @@ class GM_Combo_Regimes(GM_Endog_Error_Regimes, REGI.Regimes_Frame):
             
         name_yend.append(USER.set_name_yend_sp(self.name_y))
 
-        print(cols2regi, x_constant.shape[1], yend2.shape[1], name_x, name_yend, name_q)
         GM_Endog_Error_Regimes.__init__(
             self,
             y=y,
@@ -1590,12 +1602,12 @@ class GMM_Error_Regimes(GM_Error_Regimes, GM_Combo_Regimes, GM_Endog_Error_Regim
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -1634,7 +1646,7 @@ class GMM_Error_Regimes(GM_Error_Regimes, GM_Combo_Regimes, GM_Endog_Error_Regim
                    If True, then a spatial lag of the dependent variable is included.           
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
-                   If slx_lags>0, the specification becomes of the SDEM or GNSM type.             
+                   If slx_lags>0, the specification becomes of the SLX-Error or GNSM type.             
     vm           : boolean
                    If True, include variance-covariance matrix in summary
                    results
@@ -1654,6 +1666,9 @@ class GMM_Error_Regimes(GM_Error_Regimes, GM_Combo_Regimes, GM_Endog_Error_Regim
                    Name of dataset for use in output
     latex        : boolean
                    Specifies if summary is to be printed in latex format
+    hard_bound   : boolean
+                   If true, raises an exception if the estimated spatial
+                   autoregressive parameter is outside the maximum/minimum bounds.
     **kwargs     : keywords
                    Additional arguments to pass on to the estimators. 
                    See the specific functions for details on what can be used.
@@ -2020,7 +2035,7 @@ def _work_endog_error(
         if add_lag != False:
             model.title = "GM SPATIAL COMBO MODEL + SLX (GNSM) - REGIME %s" % r            
         else:
-            model.title = "GM SPATIALLY WEIGHTED 2SLS + SLX (SDEM) - REGIME %s" % r
+            model.title = "GM SPATIALLY WEIGHTED 2SLS + SLX (SLX-Error) - REGIME %s" % r
     model.name_ds = name_ds
     model.name_y = "%s_%s" % (str(r), name_y)
     model.name_x = ["%s_%s" % (str(r), i) for i in name_x]
